import java.util.*;
import java.awt.*;
import javax.swing.*;

// This class represents the panel that displays the Environment, Robot, Traces, Maps, etc...
public class EnvironmentPanel extends JPanel {
	public static final long serialVersionUID = 1234567890;
	
	public static final Color	POLYGON_COLOR = new Color(0, 125, 0);
	public static final Color	POLYGON_FILL_COLOR = new Color(100, 100, 255);
	public static final Color	GROWN_POLYGON_COLOR = new Color(200, 200, 200);
	public static final Color	GROWN_POLYGON_FILL_COLOR = new Color(230, 230, 255);
	private boolean 	showRobot;
	private boolean 	showMap;
	private boolean 	showBinary;
	private boolean 	showEdges;
	private boolean 	showPolygons;
	private boolean 	showSupports;
	private boolean 	showVisibility;
	private boolean 	showPath;
	private boolean 	showDestination;
	private boolean 	showGrown;
	private boolean 	showLocalization;

	private Environment		environment;


	public EnvironmentPanel(Environment env, int w, int h) {
		environment = env;

		setBackground(Color.white);
		setSize(w, h);
	}

	public void update(/*boolean sSolid, boolean sOutline, */boolean sRobot, boolean sMap, boolean sBinary,
					   boolean sEdges, boolean sPoly, boolean sSupports, boolean sVisibility, boolean sPath,
					   boolean sDest, boolean sGrown, boolean sLocalization) {
		showRobot = sRobot;
		showMap = sMap;
		showBinary = sBinary;
		showEdges = sEdges;
		showPolygons =sPoly;
		showSupports = sSupports;
		showVisibility = sVisibility;
		showPath = sPath;
		showDestination = sDest;
		showGrown = sGrown;
		showLocalization = sLocalization;
		repaint();
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		// Show the processed map data
		if (showMap || showBinary || showEdges)
			g.drawImage(environment.getMapImage(), 0, 0, this);

		// Show the GROWN polygonal model if so desired
		if (showGrown) {
			for (Polygon p: environment.getMap().getGrownPolygons()) {
				// Draw the filled polygon (reverse the y coordinates)
				g.setColor(GROWN_POLYGON_FILL_COLOR);
				Polygon temp = new Polygon();
				for (int i=0; i<p.npoints; i++)
					temp.addPoint(p.xpoints[i], environment.getHeightInPixels()- p.ypoints[i]);
				g.fillPolygon(temp);
				g.setColor(GROWN_POLYGON_COLOR);
				g.drawPolygon(temp);

				for (int i=0; i<p.npoints; i++) {
					g.setColor(Color.WHITE);
					g.fillOval(p.xpoints[i] - 2, environment.getHeightInPixels()- p.ypoints[i] - 2, 5, 5);
					g.setColor(POLYGON_COLOR);
					g.drawOval(p.xpoints[i] - 2, environment.getHeightInPixels()- p.ypoints[i] - 2, 5, 5);
					//g.drawString("("+p.xpoints[i]+","+p.ypoints[i]+")", p.xpoints[i], environment.getHeightInPixels()- p.ypoints[i]);
					//g.drawString(""+i, p.xpoints[i], environment.getHeightInPixels()- p.ypoints[i]);
				}
			}
		}

		// Show the polygonal model if so desired
		if (showPolygons) {
			for (Polygon p: environment.getMap().getPolygons()) {
				// Draw the filled polygon (reverse the y coordinates)
				g.setColor(POLYGON_FILL_COLOR);
				Polygon temp = new Polygon();
				for (int i=0; i<p.npoints; i++) {
					temp.addPoint(p.xpoints[i], environment.getHeightInPixels()- p.ypoints[i]);
				}
				g.fillPolygon(temp);
				g.setColor(POLYGON_COLOR);
				g.drawPolygon(temp);

				/*g.drawLine(p.xpoints[p.npoints-1], environment.getHeightInPixels()- p.ypoints[p.npoints-1], p.xpoints[0], environment.getHeightInPixels()- p.ypoints[0]);
				for (int i=1; i<p.npoints; i++)
					g.drawLine(p.xpoints[i-1], environment.getHeightInPixels()- p.ypoints[i-1], p.xpoints[i], environment.getHeightInPixels()- p.ypoints[i]);*/

				for (int i=0; i<p.npoints; i++) {
					g.setColor(Color.WHITE);
					g.fillOval(p.xpoints[i] - 2, environment.getHeightInPixels()- p.ypoints[i] - 2, 5, 5);
					g.setColor(POLYGON_COLOR);
					g.drawOval(p.xpoints[i] - 2, environment.getHeightInPixels()- p.ypoints[i] - 2, 5, 5);
					//g.drawString("("+p.xpoints[i]+","+p.ypoints[i]+")", p.xpoints[i], environment.getHeightInPixels()- p.ypoints[i]);
					//g.drawString(""+i, p.xpoints[i], environment.getHeightInPixels()- p.ypoints[i]);
				}
			}
		}

		// Now draw the environment on top
		//environment.draw(g, showSolid, showOutline);

		if (showRobot)
			environment.getRobot().drawWith(g, environment);

		// Show the raw map data
		for (Trace t: environment.getTraces()) {
			t.draw(g, Color.red, environment, environment.getRobot());
			int numMaps = t.getObstacles().size();
			for (int i=0; i<numMaps; i++)
				t.getObstacleSet(i).draw(g, environment);
		}

		// Show the Visibility graph if desired
		if (showVisibility && environment.getMap().getGraph() != null) {
			environment.getMap().getGraph().draw(g, environment.getHeightInPixels());
		}
		// Show the Shortest Path if desired
		if (showPath && environment.getMap().getGraph() != null) {
			environment.getMap().getGraph().drawSelectedOnly(g, environment.getHeightInPixels());
		}

		// Show the Visible support lines if desired
		if (showSupports && environment.getMap().getPolygons().size() != 0) {
			PathPlanner pp = environment.getMap().getPathPlanner();
			int x = (int)environment.getRobot().getPose().x * Environment.PIXELS_PER_CM();
			int y = (int)environment.getRobot().getPose().y * Environment.PIXELS_PER_CM();

			pp.setStart(new Point(x,y));
			ArrayList<Point> supports = pp.computeSupportPointsFrom(environment.getMap().getPolygons(), x, y);
			for (Point p: supports) {
				g.setColor(Color.RED);
				g.drawLine(x, environment.getHeightInPixels() - y, p.x, environment.getHeightInPixels() - p.y);
				g.setColor(Color.WHITE);
				g.fillOval(p.x-2, environment.getHeightInPixels() - p.y-2, 5, 5);
				g.setColor(Color.BLACK);
				g.drawOval(p.x-2, environment.getHeightInPixels() - p.y-2, 5, 5);
			}
		}
		if (showDestination)
			environment.getRobot().drawWithPose(g, environment.getDestination(), environment, Color.gray);

		// Show the localization estimates if so desired
		if (showLocalization) {
			ArrayList<Pose> estimates = environment.getMap().getLocalizer().getPoseEstimates();
			for (Pose p: estimates) {
				p.drawWith(g, Environment.PIXELS_PER_CM(), environment.getHeightInPixels());
			}
		}
	}

	public void setEnvironment(Environment e) {
		environment = e;
		repaint();
	}
	public Environment getEnvironment() { return environment;}

	public void resizeToEnvironment() {
		setSize(environment.getWidthInPixels(), environment.getHeightInPixels());
		repaint();
	}
}