import java.awt.*;

public class GridBot extends DifferentialWheelRobot {
	// Dimensions of the robot
	public static double	B_LENGTH = 11;			// length of robot body (cm)
	public static double	B_WIDTH = 8.2;			// width of robot body (cm)
	public static double	W_WIDTH = 2.2;			// width of main wheels (cm)
	public static double	W_DIAMETER = 5;			// diameter main wheels (cm)
	public static double	BW_DIAMETER = 5.6;		// diameter back wheel (cm)

	public GridBot() {
		super();
		wheelSeparation = 9.1;
	}

	public void drawWithPose(Graphics g, boolean solid, Pose p, Environment e, Color c) {
		double[]  x = new double[18];
		double[]  y = new double[18];

		// Here are the points for the robot body
		x[0] = -B_WIDTH/2;
		y[0] = -B_LENGTH/2;
		x[1] = -B_WIDTH/2;
		y[1] = B_LENGTH/2;
		x[2] = B_WIDTH/2;
		y[2] = B_LENGTH/2;
		x[3] = B_WIDTH/2;
		y[3] = -B_LENGTH/2;

		// Here are the points for the robot's left wheel
		x[4] = -B_WIDTH/2 - W_WIDTH;
		y[4] = B_LENGTH/3;
		x[5] = -B_WIDTH/2 - W_WIDTH;
		y[5] = -W_DIAMETER/3;
		x[6] = -B_WIDTH/2;
		y[6] = -W_DIAMETER/3;
		x[7] = -B_WIDTH/2;
		y[7] = B_LENGTH/3;

		// Here are the points for the robot's right wheel
		x[8] = B_WIDTH/2;
		y[8] = B_LENGTH/3;
		x[9] = B_WIDTH/2;
		y[9] = -W_DIAMETER/3;
		x[10] = B_WIDTH/2 + W_WIDTH;
		y[10] = -W_DIAMETER/3;
		x[11] = B_WIDTH/2 + W_WIDTH;
		y[11] = B_LENGTH/3;

		// Here are the points for the robot's back wheel
		x[12] = -W_WIDTH/2;
		y[12] = +B_LENGTH + BW_DIAMETER/2;
		x[13] = -W_WIDTH/2;
		y[13] = +B_LENGTH - BW_DIAMETER/2;
		x[14] = W_WIDTH/2;
		y[14] = +B_LENGTH - BW_DIAMETER/2;
		x[15] = W_WIDTH/2;
		y[15] = +B_LENGTH + BW_DIAMETER/2;

		// Here are the points for arrow lines
		y[12] = 0;
		x[12] = 0;
		y[13] = B_LENGTH/2;
		x[13] = 0;
		
		y[14] = B_LENGTH/2;
		x[14] = 0;
		y[15] = B_LENGTH/3;
		x[15] = -B_WIDTH/5;
		
		y[16] = B_LENGTH/2;
		x[16] = 0;
		y[17] = B_LENGTH/3;
		x[17] = B_WIDTH/5;

		// Rotate all points according to the robot's angle
		for (int i=0; i<=17; i++) {
			double a = Math.toRadians(p.angle-90);
			double newX = x[i]*Math.cos(a) - y[i]*Math.sin(a);
			y[i] = x[i]*Math.sin(a) + y[i]*Math.cos(a);
			x[i] = newX;
			x[i] *= SCALE;
			y[i] *= SCALE;
		}

		// Create the polygons for the body and wheels
		Polygon body = new Polygon();
		for (int i=0; i<4; i++)
			body.addPoint((int)((p.x + x[i])*Environment.PIXELS_PER_CM()), e.getHeightInPixels() - (int)((p.y + y[i])*Environment.PIXELS_PER_CM()));
		Polygon lWheel = new Polygon();
		for (int i=4; i<8; i++)
			lWheel.addPoint((int)((p.x + x[i])*Environment.PIXELS_PER_CM()), e.getHeightInPixels() - (int)((p.y + y[i])*Environment.PIXELS_PER_CM()));
		Polygon rWheel = new Polygon();
		for (int i=8; i<12; i++)
			rWheel.addPoint((int)((p.x + x[i])*Environment.PIXELS_PER_CM()), e.getHeightInPixels() - (int)((p.y + y[i])*Environment.PIXELS_PER_CM()));

		// Now Draw the body
		if (solid) {
			g.setColor(Color.lightGray);
			g.fillPolygon(body);
			g.setColor(Color.gray);
			g.fillPolygon(lWheel);
			g.fillPolygon(rWheel);
			g.setColor(Color.BLACK);
			for (int i=12; i<=16; i+=2) {
				g.drawLine((int)((p.x + x[i])*Environment.PIXELS_PER_CM()),
							e.getHeightInPixels() - (int)((p.y + y[i])*Environment.PIXELS_PER_CM()),
							(int)((p.x + x[i+1])*Environment.PIXELS_PER_CM()),
							e.getHeightInPixels() - (int)((p.y + y[i+1])*Environment.PIXELS_PER_CM()));
			}
			g.setColor(Color.black);
		}
		else {
			g.setColor(c);
		}
		g.drawPolygon(body);
		g.drawPolygon(lWheel);
		g.drawPolygon(rWheel);
		for (int i=12; i<=16; i+=2) {
			g.drawLine((int)((p.x + x[i])*Environment.PIXELS_PER_CM()),
						e.getHeightInPixels() - (int)((p.y + y[i])*Environment.PIXELS_PER_CM()),
						(int)((p.x + x[i+1])*Environment.PIXELS_PER_CM()),
						e.getHeightInPixels() - (int)((p.y + y[i+1])*Environment.PIXELS_PER_CM()));
		}

		// draw the location
		/*g.setColor(Color.blue);
		g.fillOval((int)(p.x*e.PIXELS_PER_CM())-2, e.getHeightInPixels() - (int)(p.y*e.PIXELS_PER_CM())-2, 5, 5);
		g.setColor(Color.black);
		g.drawOval((int)(p.x*e.PIXELS_PER_CM())-2, e.getHeightInPixels() - (int)(p.y*e.PIXELS_PER_CM())-2, 5, 5);*/

		// Draw the radius around the robot
		/*g.drawOval((int)((p.x - getDiameter()/2)*e.PIXELS_PER_CM()),
					e.getHeightInPixels() - (int)((p.y + getDiameter()/2)*e.PIXELS_PER_CM()),
				   (int)(getDiameter()*e.PIXELS_PER_CM()),
				   (int)(getDiameter()*e.PIXELS_PER_CM()));*/
	}

	public float getDiameter() {
		return 17 * SCALE;
	}
}