import java.awt.*;

public class Pose extends Position {
	double	time;			// time at this pose
	double	angle;			// robot angle in degrees
	double	steeringAngle;	// robot steering angle in radians
	double	iccY;			// y-coord of crcle center defining the instantaneous center of curvature

	public Pose() {
		this(0.0, 0.0, 0.0, 0.0);
	}
	public Pose(double t, double x, double y, double angle) {
		super(x, y);
		time = t;
		this.angle = angle;
	}
	public Pose(double x, double y, double angle) {
		super(x, y);
		time = 0;
		this.angle = angle;
	}
	public Pose(Pose p) {
		super(p.x, p.y);
		time = p.time;
		this.angle = p.angle;
	}

	public String toString() {
		return "Pose @" + time + " sec: (" + x + "," + y + "), " + angle + " degrees";
	}

	public void drawWith(Graphics g, int pixelsPerCM, int height) {
		// Draw the orientation
		g.setColor(Color.black);
		g.drawLine((int)x, height-(int)y, (int)(x+6*Math.cos(Math.toRadians(angle))), height-(int)(y+6*Math.sin(Math.toRadians(angle))));

		// Draw the pose with an intensity reflecting its certainty
		/*if (time > 0.5)
			g.setColor(new Color(55 + (int)(time*200), 0, 0));
		else {
			int intensity = 55 + 200 - (int)(time*200);
			g.setColor(new Color(intensity, intensity, intensity));
		}*/
		g.setColor(Color.RED);
		g.fillOval((int)x-2, height - (int)y-2, 5, 5);
		g.setColor(Color.black);
		g.drawOval((int)x-2, height - (int)y-2, 5, 5);
	}
}