import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class RTMSView extends JPanel implements ActionListener {
	public static final long serialVersionUID = 1234567890;
	
	private JMenuBar			menuBar;				// The menu bar with all the options
	private JMenu				traceMenu;				// The trace menu
	private JMenu				mapMenu;				// The map menu
	private JMenuItem			resetPoseButton;		// The button for resetting the robot's pose
	private JMenuItem			connectButton;			// The button for connecting a robot
	//private JMenuItem[]			envButtons;				// The buttons for selecting an environment
	private JMenuItem[]			robotButtons;			// The buttons for selecting a robot
	private JMenuItem[][]		traceButtons;			// The buttons for various trace-drawing parameters
	private JMenuItem[][]		mapButtons;				// The buttons for various map-drawing parameters
	private JMenuItem			clearTracesButton;		// Button used to reset all traces by clearing the data
	private JMenuItem			clearMapsButton;		// Button used to reset the maps by clearing the data
	private JMenuItem			saveGridMapButton;		// Button used to save a grid map to a file
	private JMenuItem			loadGridMapButton;		// Button used to load a grid map from a file
	private JMenuItem			saveVectorMapButton;	// Button used to save a vector map to a file
	private JMenuItem			loadVectorMapButton;	// Button used to load a vector map from a file
	private JSlider				scaleSlider;			// Slider to set robot scale
	private JSlider				thresholdSlider;		// Slider to set binary threshold of map
	private JSlider				errorThresholdSlider;	// Slider to set polygon line-fitting error threshold
	private JSlider				roundnessSlider;		// Slider to set polygon corner roundness for growing obstacles
	private JCheckBoxMenuItem	showRobotButton;		// Button used to hide or show the robot
	private JCheckBoxMenuItem	showMapButton;			// Button used to hide or show the map
	private JCheckBoxMenuItem	showBinaryButton;		// Button used to hide or show the binary map
	private JCheckBoxMenuItem	showEdgesButton;		// Button used to hide or show the edges of the map objects
	private JCheckBoxMenuItem	showPolygonsButton;		// Button used to hide or show the polygons computed from the map
	private JCheckBoxMenuItem	showDestinationButton;	// Button used to hide or show the robot's destination
	private JCheckBoxMenuItem	showSupportsButton;		// Button used to hide or show the support lines
	private JCheckBoxMenuItem	showVisibilityButton;	// Button used to hide or show the visibility graph
	private JCheckBoxMenuItem	showPathButton;			// Button used to hide or show the shortest path
	private JCheckBoxMenuItem	showTurnPathButton;		// Button used to hide or show the shortest TURN path
	private JCheckBoxMenuItem	showGrownButton;		// Button used to hide or show the grown obstacle space
	private JCheckBoxMenuItem	showLocalizationButton;	// Button used to hide or show the localization estimates

	//private JMenuItem	tempButton1;	// Button used to calculate an estimate
	//private JMenuItem	tempButton2;	// Button used to calculate an estimate
	//private JMenuItem	tempButton3;	// Button used to calculate an estimate

	private EnvironmentPanel	environmentPanel;	// displays the environment, robot, traces, maps, etc...
	private JTextField 			statusField;		// displays information in the bottom of the window

	private PCCommunicator		robotCommunicator;

	public RTMSView(JFrame owner, Environment environment, PCCommunicator communicator) {
		robotCommunicator = communicator;

		environmentPanel = new EnvironmentPanel(environment, environment.getWidthInPixels(), environment.getHeightInPixels());

		// Create the menu bar and menus
		menuBar = new JMenuBar();
		JMenu	robotMenu = new JMenu("Robot"); menuBar.add(robotMenu);
		//JMenu 	envMenu = new JMenu("Environment");
		//menuBar.add(envMenu);
		traceMenu = new JMenu("Traces");	menuBar.add(traceMenu);
		mapMenu = new JMenu("GridMap");		menuBar.add(mapMenu);
		JMenu vectorMenu = new JMenu("VectorMap");	menuBar.add(vectorMenu);
		JMenu pathPlanMenu = new JMenu("PathPlanning");	menuBar.add(pathPlanMenu);
		JMenu localizationMenu = new JMenu("Localization");	menuBar.add(localizationMenu);

		// Prepare the Traces menu items
		clearTracesButton = new JMenuItem("Clear All Traces");

		// Fill up the Robot menu
		//JMenu	robotTypeMenu = new JMenu("Type");
		//robotButtons = new JMenuItem[2];
		//robotTypeMenu.add(robotButtons[0] = new JMenuItem("CompactBot"));
		//robotTypeMenu.add(robotButtons[1] = new JMenuItem("AckermanBot"));
		//robotMenu.add(robotTypeMenu);
		robotMenu.add(showRobotButton = new JCheckBoxMenuItem("Show Robot"));
		robotMenu.add(resetPoseButton = new JMenuItem("Reset Pose"));
		showRobotButton.setSelected(true);
		scaleSlider = new JSlider(JSlider.HORIZONTAL, -200, 200, 0);

		JMenu subMenu = new JMenu("Adjust Robot Scale");
			subMenu.add(scaleSlider);
			robotMenu.add(subMenu);
		robotMenu.add(new JSeparator());

		connectButton = new JMenuItem("Connect ...");
		connectButton.setForeground(Color.white);
		connectButton.setBackground(Color.green.darker().darker());
		robotMenu.add(connectButton);

		// Fill up the Environment menu
		/*JMenu	envTypeMenu = new JMenu("Environment Type");
		envMenu.add(envTypeMenu);
		envButtons = new JMenuItem[3];
		envTypeMenu.add(envButtons[0] = new JMenuItem("Lab Floor"));
		envTypeMenu.add(envButtons[1] = new JMenuItem("Lego TestPad"));
		envTypeMenu.add(envButtons[2] = new JMenuItem("Colored Grid"));
		viewButtons = new JCheckBoxMenuItem[3];
		envMenu.add(viewButtons[0] = new JCheckBoxMenuItem("Show As Solid"));
		envMenu.add(viewButtons[1] = new JCheckBoxMenuItem("Show As Outline"));
		envMenu.add(viewButtons[2] = new JCheckBoxMenuItem("Show Robot"));
		viewButtons[0].setSelected(true);
		viewButtons[1].setSelected(true);
		viewButtons[2].setSelected(true);*/

		// Create the GridMap menu
		clearMapsButton = new JMenuItem("Clear Grid Map");
		saveGridMapButton = new JMenuItem("Save Grid Map");
		loadGridMapButton = new JMenuItem("Load Grid Map");
		showMapButton = new JCheckBoxMenuItem("Show Actual Map");
		showBinaryButton = new JCheckBoxMenuItem("Show Binary Map");
		showEdgesButton = new JCheckBoxMenuItem("Show Object Edges");
		thresholdSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 1);
		showMapButton.setSelected(true);
		mapMenu.add(loadGridMapButton);
		mapMenu.add(saveGridMapButton);
		mapMenu.add(new JSeparator());
		mapMenu.add(showMapButton);
		mapMenu.add(showBinaryButton);
		subMenu = new JMenu("Adjust Binary Threshold");
			subMenu.add(thresholdSlider);
			mapMenu.add(subMenu);
		mapMenu.add(showEdgesButton);

		// Fill up the VectorMap menu
		saveVectorMapButton = new JMenuItem("Save Vector Map");
		loadVectorMapButton = new JMenuItem("Load Vector Map");
		showPolygonsButton = new JCheckBoxMenuItem("Show Polygons");
		errorThresholdSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 10);
		vectorMenu.add(loadVectorMapButton);
		vectorMenu.add(saveVectorMapButton);
		vectorMenu.add(new JSeparator());
		vectorMenu.add(showPolygonsButton);
		subMenu = new JMenu("Adjust Error Threshold");
			subMenu.add(errorThresholdSlider);
			vectorMenu.add(subMenu);

		// Fill in the PathPlanning menu
		showDestinationButton = new JCheckBoxMenuItem("Show Destination");
		showSupportsButton = new JCheckBoxMenuItem("Show Support Lines");
		showVisibilityButton = new JCheckBoxMenuItem("Show Visibility Graph");
		showPathButton = new JCheckBoxMenuItem("Show Shortest Path");
		showTurnPathButton = new JCheckBoxMenuItem("Show Shortest Turn Path");
		showGrownButton = new JCheckBoxMenuItem("Show Grown Obstacles");
		roundnessSlider = new JSlider(JSlider.HORIZONTAL, 5, 90, 45);
		pathPlanMenu.add(showDestinationButton);
		pathPlanMenu.add(showSupportsButton);
		pathPlanMenu.add(showVisibilityButton);
		pathPlanMenu.add(showPathButton);
		pathPlanMenu.add(showTurnPathButton);
		pathPlanMenu.add(new JSeparator());
		pathPlanMenu.add(showGrownButton);
		subMenu = new JMenu("Adjust Corner Roundness");
			subMenu.add(roundnessSlider);
			pathPlanMenu.add(subMenu);

		// Fill in the Localization menu
		showLocalizationButton = new JCheckBoxMenuItem("Show Localization Estimates");
		localizationMenu.add(showLocalizationButton);

		/*tempButton1 = new JMenuItem("Get Sensor Reading ...");
		localizationMenu.add(tempButton1);
		tempButton2 = new JMenuItem("Get Foward Distance ...");
		localizationMenu.add(tempButton2);
		tempButton3 = new JMenuItem("Get Turn Amount ...");
		localizationMenu.add(tempButton3);*/

		// Set the layout and add the components
		setLayout(new BorderLayout());
		add(environmentPanel, BorderLayout.CENTER);
		statusField = new JTextField("(x, y) facing 90 degrees");
		add(statusField, BorderLayout.SOUTH);

		update();
	}

	// Lots of GET methods
	public JMenuBar getMenuBar() { return menuBar; }
	public JMenuItem getResetPoseButton() { return resetPoseButton; }
	public JMenuItem getConnectButton() { return connectButton; }
	//public JMenuItem getEnvironmentButton(int i) { return envButtons[i]; }
	//public JCheckBoxMenuItem getViewButton(int i) { return viewButtons[i]; }
	public JMenuItem getTraceButton(int i, int j) { return traceButtons[i][j]; }
	public JMenuItem getMapButton(int i, int j) { return mapButtons[i][j]; }
	public JMenuItem getClearTracesButton() { return clearTracesButton; }
	public JMenuItem getClearMapsButton() { return clearMapsButton; }
	public JMenuItem getSaveGridMapButton() { return saveGridMapButton; }
	public JMenuItem getLoadGridMapButton() { return loadGridMapButton; }
	public JMenuItem getSaveVectorMapButton() { return saveVectorMapButton; }
	public JMenuItem getLoadVectorMapButton() { return loadVectorMapButton; }
	public JMenuItem getShowMapButton() { return showMapButton; }
	public JMenuItem getRobotButton(int i) { return robotButtons[i]; }
	public EnvironmentPanel getEnvironmentPanel() { return environmentPanel; }
	public JTextField getStatusField() { return statusField; }
	public JSlider getScaleSlider() { return scaleSlider; }
	public JSlider getThresholdSlider() { return thresholdSlider; }
	public JSlider getErrorThresholdSlider() { return errorThresholdSlider; }
	public JMenuItem getShowRobotButton() { return showRobotButton; }
	public JMenuItem getShowBinaryButton() { return showBinaryButton; }
	public JMenuItem getShowEdgesButton() { return showEdgesButton; }
	public JMenuItem getShowPolygonsButton() { return showPolygonsButton; }
	public JMenuItem getShowDestinationButton() { return showDestinationButton; }
	public JMenuItem getShowSupportsButton() { return showSupportsButton; }
	public JMenuItem getShowVisibilityButton() { return showVisibilityButton; }
	public JMenuItem getShowPathButton() { return showPathButton; }
	public JMenuItem getShowTurnPathButton() { return showTurnPathButton; }
	public JMenuItem getShowGrownButton() { return showGrownButton; }
	public JSlider getRoundnessSlider() { return roundnessSlider; }
	public JMenuItem getShowLocalizationButton() { return showLocalizationButton; }

	/*public JMenuItem getTempButton1() { return tempButton1; }
	public JMenuItem getTempButton2() { return tempButton2; }
	public JMenuItem getTempButton3() { return tempButton3; }*/

	// Needed set methods
	public void setPCCommunicator(PCCommunicator com) { robotCommunicator = com; }

	// Update all the components on the main window
	public void update() {
		// Update any menu items that need a refresh
/*usbButton.setSelected(robotCommunicator instanceof USBCommunicator);
  bluetoothButton.setSelected(robotCommunicator instanceof BluetoothCommunicator);


		if (usbButton.isSelected()) {
			changeAddressButton.setEnabled(false);
			usbButton.setText("USB Connection");
		}
		else {
			changeAddressButton.setEnabled(true);
			bluetoothButton.setText("Bluetooth Connection: " + BluetoothCommunicator.ROBOT_ADDRESS);
		}
*/
        // Update the Connect Button
        connectButton.setForeground(Color.white);
        switch(robotCommunicator.getConnectionStatus()) {
        	case PCCommunicator.NOT_CONNECTED:
        		connectButton.setText("Connect ...");
        		connectButton.setBackground(new Color(0, 0, 100));
        		break;
        	case PCCommunicator.CONNECTED:
        		connectButton.setText("Waiting For Robot ...");
        		connectButton.setBackground(new Color(100, 100, 100));
        		break;
        	case PCCommunicator.READY_TO_START:
        	case PCCommunicator.START_PRESSED:
        		connectButton.setText("Start");
        		connectButton.setBackground(new Color(0, 100, 0));
        		break;
        	case PCCommunicator.RUNNING:
        		connectButton.setText("Stop");
        		connectButton.setBackground(new Color(100, 0, 0));
        }


		// Update the environment
		environmentPanel.update(/*viewButtons[0].isSelected(), viewButtons[1].isSelected(), */showRobotButton.isSelected(),
								getShowMapButton().isSelected(), getShowBinaryButton().isSelected(), getShowEdgesButton().isSelected(),
								getShowPolygonsButton().isSelected(), getShowSupportsButton().isSelected(),
								getShowVisibilityButton().isSelected(), getShowPathButton().isSelected() || getShowTurnPathButton().isSelected(),
								getShowDestinationButton().isSelected(), getShowGrownButton().isSelected(), getShowLocalizationButton().isSelected());

		// Update the status panel, showing the robot's new location and angle
		Environment model = environmentPanel.getEnvironment();
		statusField.setText(String.format("(%1.1f, %1.1f) @ %d%c",
							model.getRobot().getPose().x,
							model.getRobot().getPose().y,
							(Math.round(model.getRobot().getPose().angle)),
							(char)176));

		// Redraw the map
		model.redrawMap(showMapButton.isSelected(), showBinaryButton.isSelected(), showEdgesButton.isSelected());


		// Update the Trace and Map menu labels to show the latest pose count
		int i=0;
		for (Trace t: environmentPanel.getEnvironment().getTraces()) {
			traceButtons[i++][0].setText(t.getName() + ": " + t.getPoses().size() + " poses");
			int osNum = t.getObstacles().size();
			for (int j=0; j<osNum; j++)
				mapButtons[j][0].setText(t.getObstacleSet(j).getDescription() + ": " + t.getObstacleSet(j).getObstacles().size() + " readings");

		}
	}


	// Update the trace menu to contain viewing parameters for all traces
	public void updateTraceMenu() {
		traceMenu.removeAll();	// Clear the menu

		int numTraces = environmentPanel.getEnvironment().getTraces().size();
		traceButtons = new JMenuItem[numTraces][4];
		int i=0;
		for (Trace t: environmentPanel.getEnvironment().getTraces()) {
			// Fill up the Trace menu
			traceButtons[i][0] = new JMenuItem(t.getName() + ": " + t.getPoses().size() + " poses");
			traceButtons[i][0].setEnabled(false);
			traceMenu.add(traceButtons[i][0]);
			traceMenu.add(traceButtons[i][1] = new JCheckBoxMenuItem("   Show Plot"));
			traceMenu.add(traceButtons[i][2] = new JCheckBoxMenuItem("   Show Path"));
			traceMenu.add(traceButtons[i][3] = new JCheckBoxMenuItem("   Show Outline"));
			traceMenu.add(new JSeparator());
			traceButtons[i][1].setSelected(t.isVisible());
			traceButtons[i][2].setSelected(t.isConnected());
			traceButtons[i][3].setSelected(t.isOutlined());
			traceButtons[i][1].addActionListener(this);
			traceButtons[i][2].addActionListener(this);
			traceButtons[i][3].addActionListener(this);
			i++;
		}
		traceMenu.add(clearTracesButton);
	}

	// Update the map menu to contain viewing parameters for all obstacle sets
	public void updateMapMenu() {
		mapMenu.removeAll();	// Clear the menu
		mapMenu.add(loadGridMapButton);
		mapMenu.add(saveGridMapButton);
		mapMenu.add(new JSeparator());
		mapMenu.add(showMapButton);
		mapMenu.add(showBinaryButton);
		JMenu subMenu = new JMenu("Adjust Binary Threshold");
		subMenu.add(thresholdSlider);
		mapMenu.add(subMenu);
		mapMenu.add(showEdgesButton);
		mapMenu.add(new JSeparator());

		int numTraces = environmentPanel.getEnvironment().getTraces().size();
		if (numTraces == 0) return;
		Trace t = environmentPanel.getEnvironment().getTraces().get(0);
		int numMaps = t.getObstacles().size();
		mapButtons = new JMenuItem[numMaps][2];
		for (int i=0; i<numMaps; i++) {
			// Fill up the Trace menu
			ObstacleSet os = t.getObstacleSet(i);
			mapButtons[i][0] = new JMenuItem(os.getDescription() + ": " + os.getObstacles().size() + " readings");
			mapButtons[i][0].setEnabled(false);
			mapMenu.add(mapButtons[i][0]);
			mapMenu.add(mapButtons[i][1] = new JCheckBoxMenuItem("   Show Raw Data"));
			mapMenu.add(new JSeparator());
			mapButtons[i][1].setSelected(os.isVisible());
			mapButtons[i][1].addActionListener(this);
		}
		mapMenu.add(clearMapsButton);
	}

	// Event handler for selecting trace parameters
	public void actionPerformed(ActionEvent e) {
		int numTraces = environmentPanel.getEnvironment().getTraces().size();
		for (int i=0; i<numTraces; i++) {
			Trace t = environmentPanel.getEnvironment().getTrace(i);
			t.setVisible(traceButtons[i][1].isSelected());
			t.setShowConnected(traceButtons[i][2].isSelected());
			t.setShowRobotOutline(traceButtons[i][3].isSelected());
			int numMaps = t.getObstacles().size();
			for (int j=0; j<numMaps; j++) {
				t.getObstacleSet(j).setVisible(mapButtons[j][1].isSelected());
			}
		}
		update();
	}
}