// This class is to be written by the user to determine exactly how to apply a sensor reading to the map
public class ReadingMapper {
	private int 	width;
	private int 	height;
	private double	cells[][];	// grid cell occupancy map


	// A ReadingMapper will work on a fixed size occupancy grid with the specified width, heighth and cell array.
	public ReadingMapper(int w, int h, double[][] c) {
		width = w;
		height = h;
		cells = c;
	}


	// Apply a sensor model distribution to the reading.  This method should use the incoming parameter
	// values to spread out the single sensor reading accross multiple cells of the cells[][] array.
	// The cells array has units of pixels and corresponds directly to what you are seeing on the screen.
	// The (x,y) location and distance reading have already been converted from cm to pixels before this
	// method is called, so all units are in pixels here.
	//
	// (x,y) = robot's location (in pixels)
	// angle = robot's orientation (in degrees)
	// d = distance reading from the sensor (in pixels)
	// angleOffset = the angle offset of the sensor with respect to the robot's orientation (e.g., -90 degrees)
	// dError = the % of error associated with the distance reading (e.g., 0.08 = +/- 8%)
	// beamAngle = the full width of the sensor beam (in degrees)
	//
	public void applyReading(float x, float y, float angle, float d, float angleOffset, float dError, float beamAngle) {
		// Now apply the reading to the temporary grid
		int obX = (int)(x + (float)(d * Math.cos(Math.toRadians(angle + angleOffset))));
		int obY = (int)(y + (float)(d * Math.sin(Math.toRadians(angle + angleOffset))));
		if ((obX >= 0) && (obY >= 0) && (obX < width) && (obY < height)) {
			cells[obX][obY] += 3;
		}
	}
}


